import { broadcast, send } from './transport.js';
import { ACTIONS } from './actions.js';
import deepDiff from 'deep-diff';
import rfdc from 'rfdc';
const deepClone = rfdc();
let current = {
    clients: [],
    lagStatistics: {}
};
let clone = deepClone(current);
export function state() {
    return clone;
}
export function init(state) {
    const { clients, lagStatistics } = current;
    current = deepClone({ clients, lagStatistics, ...state });
    clone = deepClone(current);
    current.clients = [];
    current.lagStatistics = {};
    broadcast(ACTIONS.STATE_INIT, current);
}
function diffToCompressedDiff(diff) {
    const small = { k: diff.kind };
    if (diff.hasOwnProperty('path'))
        small.p = diff.path;
    if (diff.hasOwnProperty('lhs'))
        small.l = diff.lhs;
    if (diff.hasOwnProperty('rhs'))
        small.r = diff.rhs;
    if (diff.hasOwnProperty('index'))
        small.x = diff.index;
    if (diff.hasOwnProperty('item'))
        small.i = diff.item;
    return small;
}
export function update(state) {
    const { clients, lagStatistics } = current;
    const newState = { clients, lagStatistics, ...state };
    const diffs = deepDiff.diff(current, newState);
    if (diffs && diffs.length > 0) {
        diffs.map(d => deepDiff.applyChange(current, newState, d));
        broadcast(ACTIONS.STATE_UPDATE, diffs.map(diffToCompressedDiff));
        clone = deepClone(current);
    }
}
export function updateLag(id, lag) {
    const cur = state();
    cur.lagStatistics[id] = lag;
    update(cur);
}
export function addClient(id) {
    const cur = state();
    cur.clients.push(id);
    cur.lagStatistics[id] = Infinity;
    send(id, ACTIONS.STATE_INIT, cur);
    update(cur);
}
export function removeClient(id) {
    const cur = state();
    cur.clients.splice(cur.clients.indexOf(id), 1);
    delete cur.lagStatistics[id];
    update(cur);
}
